@@ -40,6 +40,7 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <glib.h>
 #include <gtk/gtk.h>
+#include <librsvg/rsvg.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-util.h>
 #include <libgnomevfs/gnome-vfs-directory.h>
@@ -202,9 +203,23 @@ nautilus_customization_data_get_next_element_for_display (NautilusCustomizationD
 
 	image_file_name = get_file_path_for_mode (data,
 						  current_file_info->name);
-	orig_pixbuf = gdk_pixbuf_new_from_file (image_file_name, NULL);	
+	orig_pixbuf = gdk_pixbuf_new_from_file (image_file_name, NULL);
+
+	if (orig_pixbuf == NULL) {
+		orig_pixbuf = rsvg_pixbuf_from_file_at_max_size (image_file_name,
+								 data->maximum_icon_width, 
+								 data->maximum_icon_height,
+								 NULL);
+	}
 	g_free (image_file_name);
 
+	if (orig_pixbuf == NULL) {
+		return nautilus_customization_data_get_next_element_for_display (data,
+										 emblem_name,
+										 pixbuf_out,
+										 label_out);
+	}
+
 	is_reset_image = eel_strcmp(current_file_info->name, RESET_IMAGE_NAME) == 0;
 
 	*emblem_name = g_strdup (current_file_info->name);
@@ -213,8 +228,8 @@ nautilus_customization_data_get_next_element_for_display (NautilusCustomizationD
 		pixbuf = nautilus_customization_make_pattern_chit (orig_pixbuf, data->pattern_frame, FALSE, is_reset_image);
 	} else {
 		pixbuf = eel_gdk_pixbuf_scale_down_to_fit (orig_pixbuf, 
-								data->maximum_icon_width, 
-								data->maximum_icon_height);
+							   data->maximum_icon_width, 
+							   data->maximum_icon_height);
 		g_object_unref (orig_pixbuf);
 	}
 	
@@ -280,8 +295,9 @@ get_global_customization_uri (const char *customization_name)
 {
 	char *directory_path, *directory_uri;
 	
-	directory_path = nautilus_make_path (NAUTILUS_DATADIR,
-					     customization_name);
+	directory_path = g_build_filename (NAUTILUS_DATADIR,
+					   customization_name,
+					   NULL);
 	directory_uri = gnome_vfs_get_uri_from_local_path (directory_path);
 	
 	g_free (directory_path);
@@ -305,8 +321,9 @@ get_private_customization_uri (const char *customization_name)
 	char *directory_path, *directory_uri;
 
 	user_directory = nautilus_get_user_directory ();
-	directory_path = nautilus_make_path (user_directory,
-					     customization_name);
+	directory_path = g_build_filename (user_directory,
+					   customization_name,
+					   NULL);
 	g_free (user_directory);
 	directory_uri = gnome_vfs_get_uri_from_local_path (directory_path);
 	g_free (directory_path);
@@ -327,7 +344,7 @@ get_file_path_for_mode (const NautilusCustomizationData *data,
 		directory_uri = get_private_customization_uri (data->customization_name);
 	}
 	
-	uri = nautilus_make_path (directory_uri, file_name);
+	uri = g_build_filename (directory_uri, file_name, NULL);
 	g_free (directory_uri);
 	directory_name = gnome_vfs_get_local_path_from_uri (uri);
 	g_free (uri);
@@ -341,6 +358,7 @@ add_reset_text (GdkPixbuf *pixbuf)
 {
 	int width, height;
 	int h_offset, v_offset;
+	PangoContext *context;
 	PangoLayout *layout;
 	PangoRectangle text_extent;
 	PangoFontDescription *font_desc;
@@ -350,13 +368,16 @@ add_reset_text (GdkPixbuf *pixbuf)
 
 	font_desc = pango_font_description_from_string ("Mono");
 
-	layout = pango_layout_new (eel_pango_ft2_get_context ());
+	context = eel_pango_ft2_get_context ();
+	layout = pango_layout_new (context);
+	g_object_unref (context);
 	pango_layout_set_text (layout, _("reset"), -1);
 	pango_layout_set_font_description (layout, font_desc);
-
+	pango_font_description_free (font_desc);
+	
 	text_extent = eel_pango_layout_fit_to_dimensions (
 		layout, width - 12, -1);
-
+	
 	/* compute text position, correcting for the imbalanced shadow, etc. */
 	h_offset = ((width - text_extent.width) / 2) - 2;
 	v_offset = (((height - 8)/ 2) - text_extent.height) / 2;
@@ -449,7 +470,7 @@ load_name_map_hash_table (NautilusCustomizationData *data)
 	data->name_map_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 	
 	/* build the path name to the browser.xml file and load it */
-	xml_path = nautilus_make_path (NAUTILUS_DATADIR, "browser.xml");
+	xml_path = g_build_filename (NAUTILUS_DATADIR, "browser.xml", NULL);
 	if (xml_path) {
 		browser_data = xmlParseFile (xml_path);
 		g_free (xml_path);
