@@ -73,7 +73,6 @@ typedef struct {
 	GnomeVFSXferOverwriteMode overwrite_mode;
 	GtkWidget *parent_view;
 	TransferKind kind;
-	gboolean show_progress_dialog;
 	void (* done_callback) (GHashTable *debuting_uris, gpointer data);
 	gpointer done_callback_data;
 	GHashTable *debuting_uris;
@@ -280,14 +279,10 @@ static void
 create_transfer_dialog (const GnomeVFSXferProgressInfo *progress_info,
 			TransferInfo *transfer_info)
 {
-	if (!transfer_info->show_progress_dialog) {
-		return;
-	}
-
 	g_return_if_fail (transfer_info->progress_dialog == NULL);
 
 	transfer_info->progress_dialog = nautilus_file_operations_progress_new 
-		(transfer_info->operation_title, "", "", "", 0, 0);
+		(transfer_info->operation_title, "", "", "", 0, 0, TRUE);
 
 	/* Treat clicking on the close box or use of the escape key
 	 * the same as clicking cancel.
@@ -307,8 +302,6 @@ create_transfer_dialog (const GnomeVFSXferProgressInfo *progress_info,
 			GTK_WINDOW (transfer_info->progress_dialog), 
 			GTK_WINDOW (gtk_widget_get_toplevel (transfer_info->parent_view)));
 	}
-
-	gtk_widget_show (GTK_WIDGET (transfer_info->progress_dialog));
 }
 
 static void
@@ -574,8 +567,7 @@ build_error_string (const char *source_name, const char *target_name,
 			break;
 		}
 		
-		if (error_string != NULL) {
-			g_assert (source_name != NULL);
+		if (error_string != NULL && source_name != NULL) {
 			result = g_strdup_printf (error_string, source_name);
 		}
 
@@ -608,8 +600,7 @@ build_error_string (const char *source_name, const char *target_name,
 			break;
 		}
 
-		if (error_string != NULL) {
-			g_assert (source_name != NULL);
+		if (error_string != NULL && source_name != NULL) {
 			result = g_strdup_printf (error_string, source_name);
 		}
 
@@ -632,8 +623,7 @@ build_error_string (const char *source_name, const char *target_name,
 			break;
 		}
 
-		if (error_string != NULL) {
-			g_assert (source_name != NULL);
+		if (error_string != NULL && source_name != NULL) {
 			result = g_strdup_printf (error_string, source_name);
 		}
 
@@ -699,8 +689,7 @@ build_error_string (const char *source_name, const char *target_name,
 				break;
 			}
 		}
-		if (error_string != NULL) {
-			g_assert (target_name != NULL);
+		if (error_string != NULL && target_name != NULL) {
 			result = g_strdup_printf (error_string, target_name);
 		}
 	}
@@ -800,6 +789,9 @@ handle_transfer_vfs_error (const GnomeVFSXferProgressInfo *progress_info,
 
 		/* transfer error, prompt the user to continue or stop */
 
+		/* stop timeout while waiting for user */
+		nautilus_file_operations_progress_pause_timeout (transfer_info->progress_dialog);
+
 		formatted_source_name = NULL;
 		formatted_target_name = NULL;
 
@@ -923,6 +915,7 @@ handle_transfer_vfs_error (const GnomeVFSXferProgressInfo *progress_info,
 				error_dialog_result = GNOME_VFS_XFER_ERROR_ACTION_SKIP;
 				break;
 			case 1:
+			case GTK_RESPONSE_DELETE_EVENT:
 				error_dialog_result = GNOME_VFS_XFER_ERROR_ACTION_ABORT;
 				break;
 			default:
@@ -957,6 +950,8 @@ handle_transfer_vfs_error (const GnomeVFSXferProgressInfo *progress_info,
 		g_free (formatted_source_name);
 		g_free (formatted_target_name);
 
+		nautilus_file_operations_progress_resume_timeout (transfer_info->progress_dialog);
+
 		return error_dialog_result;
 
 	case GNOME_VFS_XFER_ERROR_MODE_ABORT:
@@ -998,6 +993,8 @@ handle_transfer_overwrite (const GnomeVFSXferProgressInfo *progress_info,
 	int result;
 	char *text, *formatted_name;
 
+	nautilus_file_operations_progress_pause_timeout (transfer_info->progress_dialog);	
+
 	/* Handle special case files such as Trash, mount links and home directory */	
 	if (is_special_link (progress_info->target_name)) {
 		formatted_name = extract_and_ellipsize_file_name_for_dialog
@@ -1024,6 +1021,8 @@ handle_transfer_overwrite (const GnomeVFSXferProgressInfo *progress_info,
 		g_free (text);
 		g_free (formatted_name);
 
+		nautilus_file_operations_progress_resume_timeout (transfer_info->progress_dialog);
+
 		return GNOME_VFS_XFER_OVERWRITE_ACTION_SKIP;
 	}
 	
@@ -1043,6 +1042,9 @@ handle_transfer_overwrite (const GnomeVFSXferProgressInfo *progress_info,
 			(parent_for_error_dialog (transfer_info), TRUE, text, 
 			 _("Conflict while copying"),
 			 _("Replace"), _("Skip"), NULL);
+			 
+		nautilus_file_operations_progress_resume_timeout (transfer_info->progress_dialog);
+					 
 		switch (result) {
 		case 0:
 			return GNOME_VFS_XFER_OVERWRITE_ACTION_REPLACE;
@@ -1058,6 +1060,8 @@ handle_transfer_overwrite (const GnomeVFSXferProgressInfo *progress_info,
 			 _("Conflict while copying"),
 			 _("Replace All"), _("Replace"), _("Skip"), NULL);
 
+		nautilus_file_operations_progress_resume_timeout (transfer_info->progress_dialog);
+
 		switch (result) {
 		case 0:
 			return GNOME_VFS_XFER_OVERWRITE_ACTION_REPLACE_ALL;
@@ -1641,11 +1645,15 @@ static GnomeVFSURI *
 append_basename (const GnomeVFSURI *target_directory,
 		 const GnomeVFSURI *source_directory)
 {
-	const char *file_name;
+	char *file_name;
+	GnomeVFSURI *ret;
 
 	file_name = gnome_vfs_uri_extract_short_name (source_directory);
 	if (file_name != NULL) {
-		return gnome_vfs_uri_append_file_name (target_directory, file_name);
+		ret = gnome_vfs_uri_append_file_name (target_directory, 
+						      file_name);
+		g_free (file_name);
+		return ret;
 	}
 	 
 	return gnome_vfs_uri_dup (target_directory);
@@ -1687,7 +1695,6 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 	gboolean target_is_trash;
 	gboolean is_desktop_trash_link;
 	gboolean duplicate;
-	gboolean all_local;
 	
 	IconPositionIterator *icon_position_iterator;
 
@@ -1711,7 +1718,6 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 	 */
 	source_uri_list = NULL;
 	target_uri_list = NULL;
-	all_local = TRUE;
 	duplicate = copy_action != GDK_ACTION_MOVE;
 	for (p = item_uris; p != NULL; p = p->next) {
 		/* Filter out special Nautilus link files */
@@ -1725,6 +1731,9 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 		}
 
 		source_uri = gnome_vfs_uri_new ((const char *) p->data);
+		if (source_uri == NULL) {
+			continue;
+		}
 		source_dir_uri = gnome_vfs_uri_get_parent (source_uri);
 		target_uri = NULL;
 		if (target_dir != NULL) {
@@ -1749,11 +1758,6 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 			target_uri_list = g_list_prepend (target_uri_list, target_uri);
 			source_uri_list = g_list_prepend (source_uri_list, source_uri);
 
-			if (all_local && (!gnome_vfs_uri_is_local (source_uri)
-					  || !gnome_vfs_uri_is_local (target_uri))) {
-				all_local = FALSE;
-			}
-
 			if (duplicate
 			    && !gnome_vfs_uri_equal (source_dir_uri, target_dir_uri)) {
 				duplicate = FALSE;
@@ -1822,11 +1826,6 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 
 		transfer_info->kind = TRANSFER_MOVE_TO_TRASH;
 
-		/* Do an arbitrary guess that an operation will take very little
-		 * time and the progress shouldn't be shown.
-		 */
-		transfer_info->show_progress_dialog = 
-			!all_local || g_list_length ((GList *) item_uris) > 20;
 	} else if ((move_options & GNOME_VFS_XFER_REMOVESOURCE) != 0) {
 		/* localizers: progress dialog title */
 		transfer_info->operation_title = _("Moving files");
@@ -1839,11 +1838,6 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 
 		transfer_info->kind = TRANSFER_MOVE;
 
-		/* Do an arbitrary guess that an operation will take very little
-		 * time and the progress shouldn't be shown.
-		 */
-		transfer_info->show_progress_dialog = 
-			!all_local || g_list_length ((GList *) item_uris) > 20;
 	} else if ((move_options & GNOME_VFS_XFER_LINK_ITEMS) != 0) {
 		/* when creating links, handle name conflicts automatically */
 		move_options |= GNOME_VFS_XFER_USE_UNIQUE_NAMES;
@@ -1857,8 +1851,7 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 		transfer_info->cleanup_name = _("Finishing Creating Links...");
 
 		transfer_info->kind = TRANSFER_LINK;
-		transfer_info->show_progress_dialog =
-			g_list_length ((GList *)item_uris) > 20;
+
 	} else {
 		/* localizers: progress dialog title */
 		transfer_info->operation_title = _("Copying files");
@@ -1870,8 +1863,6 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 		transfer_info->cleanup_name = "";
 
 		transfer_info->kind = TRANSFER_COPY;
-		/* always show progress during copy */
-		transfer_info->show_progress_dialog = TRUE;
 	}
 
 	/* we'll need to check for copy into Trash and for moving/copying the Trash itself */
@@ -2139,7 +2130,6 @@ nautilus_file_operations_delete (const GList *item_uris,
 	uri_list = g_list_reverse (uri_list);
 
 	transfer_info = transfer_info_new (parent_view);
-	transfer_info->show_progress_dialog = TRUE;
 
 	/* localizers: progress dialog title */
 	transfer_info->operation_title = _("Deleting files");
@@ -2175,7 +2165,6 @@ do_empty_trash (GtkWidget *parent_view)
 	if (trash_dir_list != NULL) {
 		/* set up the move parameters */
 		transfer_info = transfer_info_new (parent_view);
-		transfer_info->show_progress_dialog = TRUE;
 
 		/* localizers: progress dialog title */
 		transfer_info->operation_title = _("Emptying the Trash");
@@ -2204,9 +2193,10 @@ do_empty_trash (GtkWidget *parent_view)
 static gboolean
 confirm_empty_trash (GtkWidget *parent_view)
 {
-	GtkDialog *dialog;
+	GtkWidget *dialog;
 	GtkWindow *parent_window;
 	int response;
+	GtkWidget *hbox, *image, *label, *button;
 
 	/* Just Say Yes if the preference says not to confirm. */
 	if (!eel_preferences_get_boolean (NAUTILUS_PREFERENCES_CONFIRM_TRASH)) {
@@ -2215,17 +2205,45 @@ confirm_empty_trash (GtkWidget *parent_view)
 	
 	parent_window = GTK_WINDOW (gtk_widget_get_toplevel (parent_view));
 
-	dialog = eel_show_yes_no_dialog (
-		_("Are you sure you want to permanently delete "
-		  "all of the items in the Trash?"),
-		_("Delete Trash Contents?"),
-		_("Empty"),
-		GTK_STOCK_CANCEL,
-		parent_window);
+	dialog = gtk_dialog_new ();
+	gtk_window_set_title (GTK_WINDOW (dialog), _("Empty Trash"));
+	gtk_window_set_wmclass (GTK_WINDOW (dialog), "empty_trash",
+				"Nautilus");
+	gtk_window_set_transient_for (GTK_WINDOW (dialog),
+				      GTK_WINDOW (parent_window));
 
-	gtk_dialog_set_default_response (dialog, GTK_RESPONSE_CANCEL);
+	hbox = gtk_hbox_new (FALSE, 6);
+	gtk_widget_show (hbox);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox,
+			    FALSE, FALSE, 0);
 
-	response = gtk_dialog_run (dialog);
+	image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_QUESTION,
+					  GTK_ICON_SIZE_DIALOG);
+	gtk_widget_show (image);
+	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+
+	label = gtk_label_new (_("Are you sure you want to permanently delete "
+				 "all of the items in the Trash?"));
+	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+	gtk_widget_show (label);
+	gtk_box_pack_start (GTK_BOX (hbox), label,
+			    TRUE, TRUE, 0);
+
+	gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CANCEL,
+			       GTK_RESPONSE_CANCEL);
+
+	button = eel_gtk_button_new_with_stock_icon (_("_Empty"),
+						     GTK_STOCK_DELETE);
+	gtk_widget_show (button);
+	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+
+	gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button,
+				      GTK_RESPONSE_YES);
+
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
+					 GTK_RESPONSE_YES);
+
+	response = gtk_dialog_run (GTK_DIALOG (dialog));
 
 	gtk_object_destroy (GTK_OBJECT (dialog));
 
