@@ -64,10 +64,17 @@
 #include <libnautilus-private/nautilus-file-attributes.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-icon-factory.h>
+#include <libnautilus-private/nautilus-emblem-utils.h>
 #include <libnautilus-private/nautilus-link.h>
 #include <libnautilus-private/nautilus-metadata.h>
 #include <libnautilus-private/nautilus-undo-signal-handlers.h>
+#include <libnautilus-private/egg-screen-help.h>
+#include <libnautilus-private/nautilus-mime-actions.h>
+#include <libnautilus-private/nautilus-view-identifier.h>
 #include <libnautilus/nautilus-undo.h>
+#include <libnautilus/nautilus-view.h>
+#include <bonobo/bonobo-widget.h>
+#include <bonobo/bonobo-exception.h>
 #include <string.h>
 
 static GHashTable *windows;
@@ -102,7 +109,6 @@ struct FMPropertiesWindowDetails {
 };
 
 enum {
-	PERMISSIONS_CHECKBOXES_TITLE_ROW,
 	PERMISSIONS_CHECKBOXES_OWNER_ROW,
 	PERMISSIONS_CHECKBOXES_GROUP_ROW,
 	PERMISSIONS_CHECKBOXES_OTHERS_ROW,
@@ -128,6 +134,12 @@ typedef struct {
 	FMDirectoryView *directory_view;
 } StartupData;
 
+typedef struct {
+	FMPropertiesWindow *window;
+	GtkWidget *vbox;
+	char *view_name;
+} ActivationData;
+
 
 /* drag and drop definitions */
 
@@ -143,8 +155,6 @@ static GtkTargetEntry target_table[] = {
 	{ "x-special/gnome-reset-background", 0, TARGET_RESET_BACKGROUND }
 };
 
-#define ERASE_EMBLEM_FILENAME	"erase.png"
-
 #define DIRECTORY_CONTENTS_UPDATE_INTERVAL	200 /* milliseconds */
 #define STANDARD_EMBLEM_HEIGHT			52
 #define EMBLEM_LABEL_SPACING			2
@@ -166,8 +176,10 @@ static void remove_pending_file                   (StartupData             *data
 						   gboolean                 cancel_timed_wait,
 						   gboolean                 cancel_destroy_handler);
 
+static void append_bonobo_pages                   (FMPropertiesWindow *window);
+
 GNOME_CLASS_BOILERPLATE (FMPropertiesWindow, fm_properties_window,
-			 GtkWindow, GTK_TYPE_WINDOW)
+			 GtkDialog, GTK_TYPE_DIALOG)
 
 typedef struct {
 	NautilusFile *file;
@@ -615,7 +627,35 @@ update_properties_window_title (GtkWindow *window, NautilusFile *file)
 }
 
 static void
-properties_window_file_changed_callback (GtkWindow *window, NautilusFile *file)
+clear_bonobo_pages (FMPropertiesWindow *window)
+{
+	int i;
+	int num_pages;
+	GtkWidget *page;
+
+	num_pages = gtk_notebook_get_n_pages
+				(GTK_NOTEBOOK (window->details->notebook));
+				 
+	for (i=0; i <  num_pages; i++) {
+		page = gtk_notebook_get_nth_page
+				(GTK_NOTEBOOK (window->details->notebook), i);
+
+		if (g_object_get_data (G_OBJECT (page), "is-bonobo-page")) {
+			gtk_notebook_remove_page
+				(GTK_NOTEBOOK (window->details->notebook), i);
+		}
+	}
+}
+
+static void
+refresh_bonobo_pages (FMPropertiesWindow *window)
+{
+	clear_bonobo_pages (window);
+	append_bonobo_pages (window);	
+}
+
+static void
+properties_window_file_changed_callback (FMPropertiesWindow *window, NautilusFile *file)
 {
 	g_assert (GTK_IS_WINDOW (window));
 	g_assert (NAUTILUS_IS_FILE (file));
@@ -623,7 +663,26 @@ properties_window_file_changed_callback (GtkWindow *window, NautilusFile *file)
 	if (nautilus_file_is_gone (file)) {
 		gtk_widget_destroy (GTK_WIDGET (window));
 	} else {
-		update_properties_window_title (window, file);
+		char *orig_mime_type;
+		char *new_mime_type;
+
+		orig_mime_type = nautilus_file_get_mime_type
+						(window->details->target_file);
+		nautilus_file_unref (window->details->target_file);
+
+		window->details->target_file = nautilus_file_ref (file);
+
+		update_properties_window_title (GTK_WINDOW (window), file);
+
+		new_mime_type = nautilus_file_get_mime_type
+						(window->details->target_file);
+
+		if (strcmp (orig_mime_type, new_mime_type) == 0) {
+			refresh_bonobo_pages (window);
+		}
+
+		g_free (orig_mime_type);
+		g_free (new_mime_type);
 	}
 }
 
@@ -670,13 +729,18 @@ attach_label (GtkTable *table,
 	      gboolean right_aligned,
 	      gboolean bold,
 	      gboolean ellipsize_text,
-	      gboolean selectable)
+	      gboolean selectable,
+	      gboolean mnemonic)
 {
 	GtkWidget *label_field;
 
-	label_field = ellipsize_text
-		? eel_ellipsizing_label_new (initial_text)
-		: gtk_label_new (initial_text);	
+	if (ellipsize_text) {
+		label_field = eel_ellipsizing_label_new (initial_text);
+	} else if (mnemonic) {
+		label_field = gtk_label_new_with_mnemonic (initial_text);
+	} else {
+		label_field = gtk_label_new (initial_text);
+	}
 
 	if (selectable) {
 		gtk_label_set_selectable (GTK_LABEL (label_field), TRUE);
@@ -705,7 +769,7 @@ attach_value_label (GtkTable *table,
 	      		  int column,
 	      		  const char *initial_text)
 {
-	return attach_label (table, row, column, initial_text, FALSE, FALSE, FALSE, TRUE);
+	return attach_label (table, row, column, initial_text, FALSE, FALSE, FALSE, TRUE, FALSE);
 }
 
 static GtkLabel *
@@ -714,7 +778,7 @@ attach_ellipsizing_value_label (GtkTable *table,
 				int column,
 				const char *initial_text)
 {
-	return attach_label (table, row, column, initial_text, FALSE, FALSE, TRUE, TRUE);
+	return attach_label (table, row, column, initial_text, FALSE, FALSE, TRUE, TRUE, FALSE);
 }
 
 static void
@@ -913,7 +977,7 @@ attach_option_menu (GtkTable *table,
 	return GTK_OPTION_MENU (option_menu);
 }		    	
 
-static void
+static GtkOptionMenu*
 attach_group_menu (GtkTable *table,
 		   int row,
 		   NautilusFile *file)
@@ -928,6 +992,7 @@ attach_group_menu (GtkTable *table,
 	g_signal_connect_object (file, "changed",
 				 G_CALLBACK (synch_groups_menu),
 				 option_menu, G_CONNECT_SWAPPED);	
+	return option_menu;
 }	
 
 static void
@@ -1254,16 +1319,21 @@ attach_title_field (GtkTable *table,
 		     int row,
 		     const char *title)
 {
-	return attach_label (table, row, TITLE_COLUMN, title, TRUE, TRUE, FALSE, FALSE);
+	return attach_label (table, row, TITLE_COLUMN, title, TRUE, TRUE, FALSE, FALSE, TRUE);
 }		      
 
 static guint
-append_title_field (GtkTable *table, const char *title)
+append_title_field (GtkTable *table, const char *title, GtkLabel **label)
 {
 	guint last_row;
+	GtkLabel *title_label;
 
 	last_row = append_row (table);
-	attach_title_field (table, last_row, title);
+	title_label = attach_title_field (table, last_row, title);
+
+	if (label) {
+		*label = title_label;
+	}
 
 	return last_row;
 }
@@ -1276,7 +1346,7 @@ append_title_value_pair (GtkTable *table,
 {
 	guint last_row;
 
-	last_row = append_title_field (table, title);
+	last_row = append_title_field (table, title, NULL);
 	attach_value_field (table, last_row, VALUE_COLUMN, file, file_attribute_name); 
 
 	return last_row;
@@ -1290,7 +1360,7 @@ append_title_and_ellipsizing_value (GtkTable *table,
 {
 	guint last_row;
 
-	last_row = append_title_field (table, title);
+	last_row = append_title_field (table, title, NULL);
 	attach_ellipsizing_value_field (table, last_row, VALUE_COLUMN, file, file_attribute_name);
 
 	return last_row;
@@ -1522,6 +1592,7 @@ create_basic_page (FMPropertiesWindow *window)
 	GtkWidget *button_box, *temp_button;
 	char *image_uri;
 	NautilusFile *target_file, *original_file;
+	GtkWidget *hbox, *name_label;
 
 	target_file = window->details->target_file;
 	original_file = window->details->original_file;
@@ -1534,6 +1605,16 @@ create_basic_page (FMPropertiesWindow *window)
 	window->details->basic_table = table;
 	
 	/* Icon pixmap */
+	hbox = gtk_hbox_new (FALSE, 4);
+	gtk_widget_show (hbox);
+	gtk_table_attach (table,
+			  hbox,
+			  TITLE_COLUMN, 
+			  TITLE_COLUMN + 1,
+			  0, 1,
+			  0, 0,
+			  0, 0);
+
 	icon_pixmap_widget = create_image_widget_for_file (original_file);
 	gtk_widget_show (icon_pixmap_widget);
 	
@@ -1541,13 +1622,13 @@ create_basic_page (FMPropertiesWindow *window)
 	gtk_widget_show (icon_aligner);
 
 	gtk_container_add (GTK_CONTAINER (icon_aligner), icon_pixmap_widget);
-	gtk_table_attach (table,
-			  icon_aligner,
-			  TITLE_COLUMN, 
-			  TITLE_COLUMN + 1,
-			  0, 1,
-			  0, 0,
-			  0, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), icon_aligner, TRUE, TRUE, 0);
+
+	/* Name label */
+	name_label = gtk_label_new_with_mnemonic (_("_Name:"));
+	eel_gtk_label_make_bold (GTK_LABEL (name_label));
+	gtk_widget_show (name_label);
+	gtk_box_pack_end (GTK_BOX (hbox), name_label, FALSE, FALSE, 0);
 
 	/* Name field */
 	name_field = nautilus_entry_new ();
@@ -1560,7 +1641,8 @@ create_basic_page (FMPropertiesWindow *window)
 			  0, 1,
 			  GTK_FILL, 0,
 			  0, 0);
-				   
+	gtk_label_set_mnemonic_widget (GTK_LABEL (name_label), name_field);
+
 	/* Attach parameters and signal handler. */
 	nautilus_file_ref (original_file);
 	g_object_set_data_full (G_OBJECT (name_field),"nautilus_file",
@@ -1610,14 +1692,14 @@ create_basic_page (FMPropertiesWindow *window)
 	}
 	append_title_and_ellipsizing_value (table, _("Location:"), target_file, "where");
 	if (should_show_link_target (window)) {
-		append_title_and_ellipsizing_value (table, _("Link Target:"), target_file, "link_target");
+		append_title_and_ellipsizing_value (table, _("Link target:"), target_file, "link_target");
 	}
 	if (should_show_mime_type (window)) {
-		append_title_value_pair (table, _("MIME Type:"), target_file, "mime_type");
+		append_title_value_pair (table, _("MIME type:"), target_file, "mime_type");
 	}				  
 	
 	/* Blank title ensures standard row height */
-	append_title_field (table, "");
+	append_title_field (table, "", NULL);
 	
 	append_title_value_pair (table, _("Modified:"), target_file, "date_modified");
 
@@ -1656,13 +1738,14 @@ create_basic_page (FMPropertiesWindow *window)
 static void
 create_emblems_page (FMPropertiesWindow *window)
 {
-	NautilusCustomizationData *customization_data;
 	GtkWidget *emblems_table, *button, *scroller;
-	char *emblem_name, *dot_pos;
+	char *emblem_name;
 	GdkPixbuf *pixbuf;
 	char *label;
 	NautilusFile *file;
+	GList *icons, *l;
 
+	
 	file = window->details->target_file;
 
 	/* The emblems wrapped table */
@@ -1674,29 +1757,29 @@ create_emblems_page (FMPropertiesWindow *window)
 
 	gtk_notebook_append_page (window->details->notebook, 
 				  scroller, gtk_label_new (_("Emblems")));
-	
-	/* Use nautilus_customization to make the emblem widgets */
-	customization_data = nautilus_customization_data_new ("emblems", TRUE, TRUE,
-							      NAUTILUS_ICON_SIZE_SMALL, 
-							      NAUTILUS_ICON_SIZE_SMALL);
-	
-	while (nautilus_customization_data_get_next_element_for_display (customization_data,
-									 &emblem_name,
-									 &pixbuf,
-									 &label) == GNOME_VFS_OK) {	
 
-		/* strip the suffix, if any */
-		dot_pos = strrchr(emblem_name, '.');
-		if (dot_pos) {
-			*dot_pos = '\0';
+	icons = nautilus_emblem_list_availible ();
+
+	l = icons;
+	while (l != NULL) {
+		emblem_name = l->data;
+		l = l->next;
+		
+		if (!nautilus_emblem_should_show_in_list (emblem_name)) {
+			continue;
 		}
 		
-		if (strcmp (emblem_name, "erase") == 0) {
-			g_object_unref (pixbuf);
-			g_free (label);
-			g_free (emblem_name);
+		pixbuf = nautilus_icon_factory_get_pixbuf_from_name (emblem_name, NULL,
+								     NAUTILUS_ICON_SIZE_SMALL,
+								     &label);
+
+		if (pixbuf == NULL) {
 			continue;
 		}
+
+		if (label == NULL) {
+			label = nautilus_emblem_get_keyword_from_icon_name (emblem_name);
+		}
 		
 		button = eel_labeled_image_check_button_new (label, pixbuf);
 		eel_labeled_image_set_fixed_image_height (EEL_LABELED_IMAGE (GTK_BIN (button)->child), STANDARD_EMBLEM_HEIGHT);
@@ -1707,7 +1790,7 @@ create_emblems_page (FMPropertiesWindow *window)
 
 		/* Attach parameters and signal handler. */
 		g_object_set_data_full (G_OBJECT (button), "nautilus_property_name",
-					emblem_name, g_free);
+					nautilus_emblem_get_keyword_from_icon_name (emblem_name), g_free);
 				     
 		nautilus_file_ref (file);
 		g_object_set_data_full (G_OBJECT (button), "nautilus_file",
@@ -1726,31 +1809,11 @@ create_emblems_page (FMPropertiesWindow *window)
 
 		gtk_container_add (GTK_CONTAINER (emblems_table), button);
 	}
+	eel_g_list_free_deep (icons);
+	
 	gtk_widget_show_all (emblems_table);
 }
 
-static void
-add_permissions_column_label (GtkTable *table, 
-			      NautilusFile *file,
-			      int column, 
-			      const char *title_text)
-{
-	GtkWidget *label;
-
-	label = gtk_label_new (title_text);
-	eel_gtk_label_make_bold (GTK_LABEL (label));
-	gtk_widget_set_sensitive (GTK_WIDGET (label), 
-				  nautilus_file_can_set_permissions (file));
-
-	/* Text is centered in table cell by default, which is what we want here. */
-	gtk_widget_show (label);
-	
-	gtk_table_attach_defaults (table, label,
-			  	   column, column + 1,
-			  	   PERMISSIONS_CHECKBOXES_TITLE_ROW, 
-			  	   PERMISSIONS_CHECKBOXES_TITLE_ROW + 1);
-}	
-
 static void
 update_permissions_check_button_state (GtkWidget *check_button, NautilusFile *file)
 {
@@ -1857,13 +1920,22 @@ add_permissions_checkbox (GtkTable *table,
 			  GnomeVFSFilePermissions permission_to_check)
 {
 	GtkWidget *check_button;
+	gchar *label;
 
-	check_button = gtk_check_button_new ();
+	if (column == PERMISSIONS_CHECKBOXES_READ_COLUMN) {
+		label = _("_Read");
+	} else if (column == PERMISSIONS_CHECKBOXES_WRITE_COLUMN) {
+		label = _("_Write");
+	} else {
+		label = _("E_xecute");
+	}
+
+	check_button = gtk_check_button_new_with_mnemonic (label);
 	gtk_widget_show (check_button);
 	gtk_table_attach (table, check_button,
 			  column, column + 1,
 			  row, row + 1,
-			  0, 0,
+			  GTK_FILL, 0,
 			  0, 0);
 
 	set_up_permissions_checkbox (check_button, file, permission_to_check);
@@ -1880,7 +1952,7 @@ append_special_execution_checkbox (FMPropertiesWindow *window,
 
 	last_row = append_row (table);
 
-	check_button = gtk_check_button_new_with_label (label_text);
+	check_button = gtk_check_button_new_with_mnemonic (label_text);
 	gtk_widget_show (check_button);
 
 	gtk_table_attach (table, check_button,
@@ -1927,17 +1999,17 @@ append_special_execution_flags (FMPropertiesWindow *window,
 				GtkTable *table)
 {
 	remember_special_flags_widget (window, append_special_execution_checkbox 
-		(window, table, _("Set User ID"), GNOME_VFS_PERM_SUID));
+		(window, table, _("Set _user ID"), GNOME_VFS_PERM_SUID));
 
 	window->details->first_special_flags_row = table->nrows - 1;
 
 	remember_special_flags_widget (window, GTK_WIDGET (attach_title_field 
-		(table, table->nrows - 1, _("Special Flags:"))));
+		(table, table->nrows - 1, _("Special flags:"))));
 
 	remember_special_flags_widget (window, append_special_execution_checkbox 
-		(window, table, _("Set Group ID"), GNOME_VFS_PERM_SGID));
+		(window, table, _("Set gro_up ID"), GNOME_VFS_PERM_SGID));
 	remember_special_flags_widget (window, append_special_execution_checkbox 
-		(window, table, _("Sticky"), GNOME_VFS_PERM_STICKY));
+		(window, table, _("_Sticky"), GNOME_VFS_PERM_STICKY));
 
 	remember_special_flags_widget (window, append_separator (table));
 	++window->details->num_special_flags_rows;
@@ -1960,6 +2032,8 @@ create_permissions_page (FMPropertiesWindow *window)
 	NautilusFile *file;
 	guint last_row;
 	guint checkbox_titles_row;
+	GtkLabel *group_label;
+	GtkOptionMenu *group_menu;
 
 	file = window->details->target_file;
 
@@ -1982,7 +2056,7 @@ create_permissions_page (FMPropertiesWindow *window)
 				    GTK_WIDGET (page_table), 
 				    TRUE, TRUE, 0);
 
-		attach_title_field (page_table, last_row, _("File Owner:"));
+		attach_title_field (page_table, last_row, _("File owner:"));
 		if (nautilus_file_can_set_owner (file)) {
 			/* Option menu in this case. */
 			attach_owner_menu (page_table, last_row, file);
@@ -1991,35 +2065,33 @@ create_permissions_page (FMPropertiesWindow *window)
 			attach_value_field (page_table, last_row, VALUE_COLUMN, file, "owner"); 
 		}
 
-		last_row = append_title_field (page_table, _("File Group:"));
 		if (nautilus_file_can_set_group (file)) {
+			last_row = append_title_field (page_table,
+						       _("_File group:"),
+						       &group_label);
 			/* Option menu in this case. */
-			attach_group_menu (page_table, last_row, file);
+			group_menu = attach_group_menu (page_table, last_row,
+							file);
+			gtk_label_set_mnemonic_widget (GTK_LABEL (group_label),
+						       GTK_WIDGET (group_menu));
 		} else {
+			last_row = append_title_field (page_table,
+						       _("File group:"),
+						       NULL);
 			/* Static text in this case. */
 			attach_value_field (page_table, last_row, VALUE_COLUMN, file, "group"); 
 		}
 
 		append_separator (page_table);
 
-		/* This next empty label is a hack to make the title row
-		 * in the main table the same height as the title row in
-		 * the checkboxes sub-table so the other row titles will
-		 * line up horizontally with the checkbox rows.
-		 */
-		checkbox_titles_row = append_title_field (page_table, "");
-
-		append_title_field (page_table, _("Owner:"));
-		append_title_field (page_table, _("Group:"));
-		append_title_field (page_table, _("Others:"));
+		checkbox_titles_row = append_title_field (page_table, _("Owner:"), NULL);
+		append_title_field (page_table, _("Group:"), NULL);
+		append_title_field (page_table, _("Others:"), NULL);
 		
-		/* Make separate table for grid of checkboxes so it can be
-		 * homogeneous; we don't want overall table to be homogeneous though.
-		 */
 		check_button_table = GTK_TABLE (gtk_table_new 
 						   (PERMISSIONS_CHECKBOXES_ROW_COUNT, 
 						    PERMISSIONS_CHECKBOXES_COLUMN_COUNT, 
-						    TRUE));
+						    FALSE));
 		apply_standard_table_padding (check_button_table);
 		gtk_widget_show (GTK_WIDGET (check_button_table));
 		gtk_table_attach (page_table, GTK_WIDGET (check_button_table),
@@ -2028,18 +2100,6 @@ create_permissions_page (FMPropertiesWindow *window)
 				  0, 0,
 				  0, 0);
 
-		add_permissions_column_label (check_button_table, file,
-					      PERMISSIONS_CHECKBOXES_READ_COLUMN,
-					      _("Read"));
-
-		add_permissions_column_label (check_button_table, file,
-					      PERMISSIONS_CHECKBOXES_WRITE_COLUMN,
-					      _("Write"));
-
-		add_permissions_column_label (check_button_table, file,
-					      PERMISSIONS_CHECKBOXES_EXECUTE_COLUMN,
-					      _("Execute"));
-
 		add_permissions_checkbox (check_button_table, file, 
 					  PERMISSIONS_CHECKBOXES_OWNER_ROW,
 					  PERMISSIONS_CHECKBOXES_READ_COLUMN,
@@ -2089,9 +2149,9 @@ create_permissions_page (FMPropertiesWindow *window)
 
 		append_special_execution_flags (window, page_table);
 
-		append_title_value_pair (page_table, _("Text View:"), file, "permissions");		
-		append_title_value_pair (page_table, _("Number View:"), file, "octal_permissions");
-		append_title_value_pair (page_table, _("Last Changed:"), file, "date_permissions");
+		append_title_value_pair (page_table, _("Text view:"), file, "permissions");		
+		append_title_value_pair (page_table, _("Number view:"), file, "octal_permissions");
+		append_title_value_pair (page_table, _("Last changed:"), file, "date_permissions");
 		
 	} else {
 		file_name = nautilus_file_get_display_name (file);
@@ -2102,6 +2162,126 @@ create_permissions_page (FMPropertiesWindow *window)
 	}
 }
 
+static GtkWidget *
+bonobo_page_error_message (const char *view_name,
+			   const char *msg)
+{
+	GtkWidget *hbox;
+	GtkWidget *label;
+	GtkWidget *image;
+
+	hbox = gtk_hbox_new (FALSE, GNOME_PAD);
+	image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_ERROR,
+					  GTK_ICON_SIZE_DIALOG);
+
+	msg = g_strdup_printf ("There was an error while trying to create the view named `%s':  %s", view_name, msg);
+	label = gtk_label_new (msg);
+
+	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+
+	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+	gtk_widget_show_all (hbox);
+
+	return hbox;
+}
+
+static void
+bonobo_page_activate_callback (CORBA_Object obj,
+			       const char *error_reason,
+			       gpointer user_data)
+{
+	ActivationData *data;
+	FMPropertiesWindow *window;
+	GtkWidget *widget;
+	CORBA_Environment ev;
+
+	data = (ActivationData *)user_data;
+	window = data->window;
+
+	g_return_if_fail (FM_IS_PROPERTIES_WINDOW (window));
+
+	CORBA_exception_init (&ev);
+	widget = NULL;
+
+	if (obj != CORBA_OBJECT_NIL) {
+		Bonobo_Control control;
+		Nautilus_View view;
+		char *uri;
+		
+		control = Bonobo_Unknown_queryInterface
+				(obj, "IDL:Bonobo/Control:1.0", &ev);
+
+		view = Bonobo_Unknown_queryInterface
+				(control, "IDL:Nautilus/View:1.0", &ev);
+
+		uri = nautilus_file_get_uri (window->details->target_file);
+
+		Nautilus_View_load_location (view, uri, &ev);
+		
+		if (!BONOBO_EX (&ev)) {
+			widget = bonobo_widget_new_control_from_objref
+						(control, CORBA_OBJECT_NIL);
+		}
+
+		g_free (uri);
+	}
+
+	if (widget == NULL) {
+		widget = bonobo_page_error_message (data->view_name,
+						    error_reason);
+	}
+
+	gtk_container_add (GTK_CONTAINER (data->vbox), widget);
+	gtk_widget_show (widget);
+
+	g_free (data->view_name);
+	g_free (data);
+}
+
+static void
+append_bonobo_pages (FMPropertiesWindow *window)
+{
+	GList *components, *l;
+	CORBA_Environment ev;
+
+	/* find all the property page views for this file */
+	components = nautilus_mime_get_all_components_for_file_extended
+		(window->details->target_file,
+		 "nautilus:property_page_name.defined()");
+	
+	CORBA_exception_init (&ev);
+
+	l = components;
+	while (l != NULL) {
+		NautilusViewIdentifier *view_id;
+		Bonobo_ServerInfo *server;
+		ActivationData *data;
+		GtkWidget *vbox;
+
+		server = l->data;
+		l = l->next;
+
+		view_id = nautilus_view_identifier_new_from_property_page (server);
+		vbox = create_page_with_vbox (window->details->notebook,
+					      view_id->name);
+
+		/* just a tag...the value doesn't matter */
+		g_object_set_data (G_OBJECT (vbox), "is-bonobo-page",
+				  vbox);
+
+		data = g_new (ActivationData, 1);
+		data->window = window;
+		data->vbox = vbox;
+		data->view_name = g_strdup (view_id->name);
+
+		bonobo_activation_activate_from_id_async (view_id->iid,
+					0, bonobo_page_activate_callback,
+					data, &ev);
+	}
+
+}
+
 static gboolean
 should_show_emblems (FMPropertiesWindow *window) 
 {
@@ -2153,13 +2333,32 @@ startup_data_free (StartupData *data)
 	g_free (data);
 }
 
+static void
+help_button_callback (GtkWidget *widget, GtkWidget *property_window)
+{
+	GError *error = NULL;
+	char *message;
+
+	egg_screen_help_display_desktop (
+		gtk_window_get_screen (GTK_WINDOW (property_window)),
+		NULL, "user-guide", "wgosnautilus.xml", "gosnautilus-51", &error);
+
+	if (error) {
+		message = g_strdup_printf (_("There was an error displaying help: \n%s"),
+					   error->message);
+		eel_show_error_dialog (message, _("Couldn't show help"),
+				       GTK_WINDOW (property_window));
+		g_error_free (error);
+		g_free (message);
+	}
+}
+
 static FMPropertiesWindow *
 create_properties_window (StartupData *startup_data)
 {
 	FMPropertiesWindow *window;
 	GList *attributes;
 	GtkWidget *vbox;
-	GtkWidget *hbox;
 	GtkWidget *button;
 
 	window = FM_PROPERTIES_WINDOW (gtk_widget_new (fm_properties_window_get_type (), NULL));
@@ -2167,8 +2366,10 @@ create_properties_window (StartupData *startup_data)
 	window->details->original_file = nautilus_file_ref (startup_data->original_file);
 	window->details->target_file = nautilus_file_ref (startup_data->target_file);
 	
-  	gtk_container_set_border_width (GTK_CONTAINER (window), GNOME_PAD);
 	gtk_window_set_wmclass (GTK_WINDOW (window), "file_properties", "Nautilus");
+	gtk_window_set_resizable (GTK_WINDOW (window), FALSE);
+	gtk_window_set_screen (GTK_WINDOW (window),
+			       gtk_widget_get_screen (GTK_WIDGET (startup_data->directory_view)));
 
 	/* Set initial window title */
 	update_properties_window_title (GTK_WINDOW (window), window->details->target_file);
@@ -2201,8 +2402,9 @@ create_properties_window (StartupData *startup_data)
 
 	/* Create box for notebook and button box. */
 	vbox = gtk_vbox_new (FALSE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
 	gtk_widget_show (vbox);
-	gtk_container_add (GTK_CONTAINER (window), 
+	gtk_container_add (GTK_CONTAINER (GTK_DIALOG(window)->vbox), 
 			   GTK_WIDGET (vbox));
 
 	/* Create the notebook tabs. */
@@ -2221,19 +2423,26 @@ create_properties_window (StartupData *startup_data)
 	if (should_show_permissions (window)) {
 		create_permissions_page (window);
 	}
-	
-	/* Create box for close button. */
-	hbox = gtk_hbutton_box_new ();
-	gtk_widget_show (hbox);
-	gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET (hbox),
-			    FALSE, TRUE, 5);
-	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox),
-				   GTK_BUTTONBOX_END);  
 
-	/* Create close button. */
-	button = gtk_button_new_from_stock (GTK_STOCK_CLOSE);
+	/* append pages from available views */
+	append_bonobo_pages (window);
+	
+	/* Create buttons for action area. */
+	gtk_dialog_set_has_separator (GTK_DIALOG (window), FALSE);
+	gtk_button_box_set_layout (GTK_BUTTON_BOX (GTK_DIALOG (window)->action_area),
+				   GTK_BUTTONBOX_EDGE);
+
+	button = gtk_button_new_from_stock (GTK_STOCK_HELP);
  	gtk_widget_show (button);
-	gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET (button),
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window)->action_area), GTK_WIDGET (button),
+			    FALSE, TRUE, 0);
+	g_signal_connect_object (button, "clicked",
+				 G_CALLBACK (help_button_callback),
+				 window, 0);
+	
+	button = gtk_button_new_from_stock (GTK_STOCK_CLOSE);
+	gtk_widget_show (button);
+	gtk_box_pack_end (GTK_BOX (GTK_DIALOG (window)->action_area), GTK_WIDGET (button),
 			    FALSE, TRUE, 0);
 	g_signal_connect_swapped (button, "clicked",
 				  G_CALLBACK (gtk_widget_destroy),
@@ -2258,7 +2467,7 @@ get_target_file (NautilusFile *file)
 		if (nautilus_file_is_local (file)) {
 			uri = nautilus_file_get_uri (file);
 
-			switch (nautilus_link_local_get_link_type (uri)) {
+			switch (nautilus_link_local_get_link_type (uri, NULL)) {
 			case NAUTILUS_LINK_MOUNT:
 			case NAUTILUS_LINK_TRASH:
 			case NAUTILUS_LINK_HOME:
@@ -2372,6 +2581,8 @@ fm_properties_window_present (NautilusFile *original_file, FMDirectoryView *dire
 	/* Look to see if there's already a window for this file. */
 	existing_window = g_hash_table_lookup (windows, original_file);
 	if (existing_window != NULL) {
+		gtk_window_set_screen (existing_window,
+				       gtk_widget_get_screen (GTK_WIDGET (directory_view)));
 		gtk_window_present (existing_window);
 		return;
 	}
@@ -2525,6 +2736,4 @@ static void
 fm_properties_window_instance_init (FMPropertiesWindow *window)
 {
 	window->details = g_new0 (FMPropertiesWindowDetails, 1);
-
-	eel_gtk_window_set_up_close_accelerator (GTK_WINDOW (window));
 }
